%% Sloth's command line handling.
%%
%% Copyright (C) 2005-2006, Babel Group
%%
%% Written by: 
%%   * Emilio Jesús Gallego Arias
%%
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

:- module(cmdopts, [process_options/0, cmd_options/2, set_option/2, unset_option/1, help/0]).

%% This module contains the command line options database.

:- use_module(library(dynamic),   [asserta/1, retract/1]).
:- use_module(library(atom2term), [string2term/2]).

:- dynamic internal_cmd_options/2.

%% Show help.
help :-
	option(N, NP, Dv, Help),
	make_pretty_opts(NP, Dv, Opts),
	format("\t~p ~p\t\t~s\n", [N, Opts, Help]),
	fail.
help.

make_pretty_opts(0, _Val, "") :- !.
make_pretty_opts(_N, Val, Val).

%% Valid options.

is_option(Opt) :-
	option(Opt, _, _, _),
	!.

option('-v',    0, none,    "Verbose compiling").
option('-np',   0, none,    "Exclude prelude from compilation").
option('-i',    0, none,    "Start interactive shell").
option('-l',    1, ['.'],   "Library directory").
option('-tp',   0, none,    "Enable compiler profiling").
option('File',  0, none,    "File to compile").

set_option(Option, Value) :-
	is_option(Option),
	asserta(internal_cmd_options(Option, Value)).

unset_option(Option) :-
	is_option(Option),
	retract(internal_cmd_options(Option, _)).

process_options :-
	current_prolog_flag(argv, Args),
	(
	    length(Args, 0) ->
	    fail
	;
	    process_args(Args)
	).

cmd_options(Option, Value) :-
	(
	    internal_cmd_options(Option, Value) ->
	    true
	;
	    option(Option, _, Value, _Desc),
	    (
		Value = none ->
		fail
	    ;
		true
	    )
	).

%% Throws an invalid_argument exception.
process_args([]).
process_args([File]) :-
	\+ option(File, _, _, _),
	!,
	set_option('File', File).
process_args([A1|AL]) :-
	(
	    option(A1, NumParams, _, _) ->
	    get_params(AL, NumParams, NAL, Params),
	    set_option(A1, Params),
	    process_args(NAL)
	;
	    throw(invalid_arg(A1))
	).

% Incomplete support for parameters.
get_params(AL, 0, AL, []) :- 
	!.
get_params([P|PL], N, AL, [P|NPL]) :-
	N1 is N - 1,
	get_params(PL, N1, AL, NPL).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:
