%% Sloth's error handling. 
%%
%% Copyright (C) 2005-2006, Babel Group
%%
%% Written by: 
%%   * Emilio Jesús Gallego Arias
%%
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

:- module(error, [do_error/1]).

:- use_module(library(format), [format/2]).

:- use_module(debug, [debug_write/1, pretty_write/1, pretty_list_write/1]).
:- use_module(types, [pretty_type/2]).

%% we use the term error/2, that contains the handler and the
%% argument(s)
do_error(error(typer, EArgs)) :-
	do_typer_error(EArgs),
	!.

do_error(error(symbol_resolution, EArgs)) :-
	do_resolution_error(EArgs),
	!.

do_error(error(EType, EArgs)) :-
	format("Error in ~p: ~p~n", [EType, EArgs]).

error(parse_error(_Position, _Token, _Type, _Recovery)).

error(lexer_error((_Pos, _CandToken, _Input, _NewInput))).

error(trad_error(_Token, _Type)).

error(error(_, _)).

do_typer_error(type_def_mismatch(Function, Type, DefType)) :-
	pretty_type(Type, T1),
	pretty_type(DefType, T2),
	prettyvars([T1, T2]),
	format("The declared type for ~p (~p) is too general: inferred ~p~n",
	       [Function, T2, T1]
	      ).
do_typer_error(too_much_args(A)) :-
	debug_write('Too much arguments: '),
	pretty_write(A),
	nl.

do_typer_error(could_not_unify(A, B)) :-
	pretty_type(A, A1),
	pretty_type(B, B1),
	prettyvars([A1, B1]),
	format("Could not unify type ~p with ~p~n", [A1, B1]).

do_resolution_error(symbol_undefined(qid(S))) :-
	format("Symbol ~p is undefined~n", [S]).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

