:- module(scope, [open_scope/1, close_scope/0, close_named_scope/1,
	          lookup_symbol/2, lookup_symbol_unref/2,
	          insert_symbol/2, 
		  scope_name/1, show_scope/0, 
	          new_scope/1, delete_scope/1]).

:- use_module(library(dynamic),  [asserta/1, retract/1, retractall/1]).
:- use_module(library(dict),     [dic_lookup/3, dic_get/3, dic_replace/4] ).

%% We have two level scopes. This is needed so we are able to use
%% the compiler as a library.

:- data current_scope/1.
:- data scope/3.

new_scope(Name) :-
	asserta(current_scope(Name)),
	!.

delete_scope(Name) :-
	retract(current_scope(Name)),
	retractall(scope(Name, _, _)),
	!.

%% 
open_scope(Name) :-
	current_scope(Sn),
	asserta(scope(Sn, Name, _Dict)),
	!.
%% 
close_scope :-
	current_scope(Sn),
	scope(Sn, Name, _Dict),
	!,
	retract(scope(Sn, Name, _)),
	!.

close_named_scope(Name) :-
	current_scope(Sn),
	retractall(scope(Sn, Name, _)),
	!.

%% !!!! Beware all the cuts here are red.
lookup_symbol(Name, Object) :-
	current_scope(Sn),
	scope(Sn, _, Dicc),
	dic_get(Dicc, Name, Object).

lookup_symbol_unref(Name, Object) :-
	current_scope(Sn),
	scope(Sn, _, Dicc),
	dic_get(Dicc, Name, Object),
	!.

lookup_symbol_unref(Name, unref(Name)) :-
	!.

%% Strange implementation was this.
% lookup_symbol_unref(Name, unref(Name)) :-
% 	retract(scope(MName, Dicc)),
% 	!,
% 	dic_lookup(Dicc, Name, unref(Name)),
% 	asserta(scope(MName, Dicc)).
	
%%

insert_symbol(Name, Object) :-
	current_scope(Sn),
	scope(Sn, SName, Dicc),
	!,
	(
	    dic_lookup(Dicc, Name, Object),
	    Dicc = Dicc2
	;
	    dic_replace(Dicc, Name, Object, Dicc2)
	),
	retract(scope(Sn, SName, _)),
	asserta(scope(Sn, SName, Dicc2)).

%%
show_scope :- 
	current_scope(Sn),
	findall((N,D), (
			   scope(Sn, N, D),
			   N \= imported('Prelude')
		       ), L),
	write(L),
	nl.

scope_name(Name) :-
	current_scope(Sn),
	!,
	scope(Sn, Name, _).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

